@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,21 +12,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
 #
-#      http://www.apache.org/licenses/LICENSE-2.0
+#     http://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
 """Utilities we could consider upstreaming to Jax.
 """
 
@@ -42,7 +40,7 @@ from jax.interpreters import partial_eval as pe
 from jax.interpreters import xla
 import jax.lib.xla_bridge as xb
 import jax.numpy as jnp
-import numpy as onp
+import numpy as np
 
 
 def _replicate(x, devices=None):
@@ -141,6 +139,8 @@ def prefetch_to_device(iterator, size, devices=None):
   This utility takes an iterator and returns a new iterator which fills an on
   device prefetch buffer. Eager prefetching can improve the performance of
   training loops significantly by overlapping compute and data transfer.
+  
+  This utility is mostly useful for GPUs, for TPUs it should not be necessary.
 
   Args:
     iterator: an iterator that yields a pytree of ndarrays where the first
@@ -227,10 +227,10 @@ def scan_in_dim(body_fn, init, xs, axis=(0,), keepdims=False):
     axis = (axis,)
 
   def transpose_in(x):
-    perm = axis + tuple(onp.delete(onp.arange(x.ndim), axis))
+    perm = axis + tuple(np.delete(np.arange(x.ndim), axis))
     return x.transpose(perm)
   def transpose_out(x):
-    perm = axis + tuple(onp.delete(onp.arange(x.ndim), axis))
+    perm = axis + tuple(np.delete(np.arange(x.ndim), axis))
     return x.transpose(_invert_perm(perm))
 
   def body_wrapper(c, xs):
