@@ -25,7 +25,7 @@ from .attention import (MultiHeadDotProductAttention, SelfAttention,
                         make_causal_mask, combine_masks)
 from ..core import broadcast
 from .linear import Conv, ConvTranspose, Dense, DenseGeneral, Embed
-from .module import Module, compact, enable_named_call, disable_named_call, Variable
+from .module import Module, compact, enable_named_call, disable_named_call, Variable, init, init_with_output, apply
 from .normalization import BatchNorm, GroupNorm, LayerNorm
 from .pooling import avg_pool, max_pool
 from .recurrent import GRUCell, LSTMCell, ConvLSTM, OptimizedLSTMCell
