@@ -30,9 +30,11 @@ from jax import tree_util
 import numpy as np
 
 import flax
+from flax import errors
 from flax import traverse_util
 from flax import serialization
-from flax.core import Scope, apply
+from flax import core
+from flax.core import Scope
 from flax.core.scope import CollectionFilter, Variable, VariableDict, FrozenVariableDict, union_filters
 from flax.core.frozen_dict import FrozenDict, freeze
 
@@ -50,14 +52,6 @@ _CallableT = TypeVar('_CallableT', bound=Callable)
 
 # pylint: disable=protected-access,attribute-defined-outside-init
 
-def _check_omnistaging():
-  if not jax.config.omnistaging_enabled:
-    raise RuntimeError(
-        "Flax linen API requires JAX omnistaging to be enabled:\n"
-        "  from jax.config import config\n"
-        "  config.enable_omnistaging()")
-
-
 def _indent(x: str, num_spaces: int):
   indent_str = ' ' * num_spaces
   lines = x.split('\n')
@@ -270,7 +264,7 @@ def wrap_method_once(fun: Callable[..., Any]) -> Callable[..., Any]:
 
     if is_compact_method:
       if self.scope is None:
-        raise ValueError("Can't call compact methods on unbound modules")
+        raise errors.CallCompactUnboundModuleError()
       self._state.in_compact_method = True
     _context.module_stack.append(self)
     try:
@@ -302,8 +296,9 @@ def _wrap_hash(hash_fn: Callable[..., Any]) -> Callable[..., Any]:
 def _get_unbound_fn(method_or_fn: Callable[..., Any]) -> Callable[..., Any]:
   """Returns an unbound function from a method that is possibly bound.
   
-  This means that the returned function does no longer depend on the instance
-  of the class, which is passed as it first argument. 
+  This means that if the passed function belongs of an instance of a class, then
+  the returned function does no longer depend on the instance, which is passed
+  as the first argument to the function.
 
   Args:
     method_or_fn: A class method or function.
@@ -311,12 +306,16 @@ def _get_unbound_fn(method_or_fn: Callable[..., Any]) -> Callable[..., Any]:
     An unbound version of input function.
   """
   if inspect.ismethod(method_or_fn):
-    return method_or_fn.__func__  # pytype: disable=attribute-error
-  elif callable(method_or_fn):
-    return method_or_fn
-  else:
-    raise ValueError('Expect a function or method.')
+    method_or_fn = method_or_fn.__func__  # pytype: disable=attribute-error
 
+  # The method should be callable, and it should have at least one argument
+  # representing the class that is passed in.
+  if (not callable(method_or_fn) or 
+      len(inspect.signature(method_or_fn).parameters) < 1):
+    raise errors.ApplyModuleInvalidMethodError(method_or_fn)
+  
+  return method_or_fn
+  
 
 @dataclasses.dataclass
 class _ModuleInternalState:
@@ -329,11 +328,16 @@ class _ModuleInternalState:
   in_compact_method: bool = False
   in_setup: bool = False
   setup_called: bool = False
+  is_initialized: bool = False
   autoname_cursor: Optional[dict] = dataclasses.field(default_factory=dict)
   children: Dict[str, Union[str, 'Module']] = dataclasses.field(default_factory=dict)
 
   def reset(self):
-    """Resets transient state."""
+    """Resets transient state. 
+    
+    This function is called after each module method, so only attributes that
+    are method-dependent are reset.
+    """
     self.in_compact_method = False
     self.in_setup = False
     self.autoname_cursor = dict()
@@ -344,6 +348,7 @@ class _ModuleInternalState:
       in_compact_method=self.in_compact_method,
       in_setup=self.in_setup,
       setup_called=False,  # setup_called is object local, not shared.
+      is_initialized=self.is_initialized,
       autoname_cursor=dict(self.autoname_cursor))
     return cloned
 
@@ -351,6 +356,7 @@ class _ModuleInternalState:
     """Re-imports transform-preserved state from across transform boundary."""
     self.in_compact_method = other.in_compact_method
     self.in_setup = other.in_setup
+    self.is_initialized = other.is_initialized
     self.autoname_cursor = dict(other.autoname_cursor)
 
 _uninitialized_module_internal_state = _ModuleInternalState()
@@ -435,8 +441,7 @@ class Module:
     # Use cls.__dict__ to get annotations of cls itself (no parent class).
     annotations = dict(cls.__dict__.get('__annotations__', {}))
     if 'parent' in annotations or 'name' in annotations:
-      raise ValueError(
-          f'properties `parent` and `name` are reserved: {annotations}')
+      raise errors.ReservedModuleAttributeError(annotations)
     # Add `parent` and `name` default fields at end.
     # We temporarily modify base class __dataclass_fields__ to force desired
     # argument behavior and ordering from dataclass class-transform.
@@ -467,10 +472,7 @@ class Module:
     n_compact_fns = len([method_name for method_name in methods
                          if hasattr(getattr(cls, method_name), 'compact')])
     if n_compact_fns > 1:
-      raise RuntimeError(
-          'Only one method per class can be @compact. You can remove @compact '
-          'and define submodules and variables in setup(), or use two '
-          'separate modules.')
+      raise errors.MultipleMethodsCompactError()
 
   @classmethod
   def _wrap_module_methods(cls):
@@ -491,7 +493,7 @@ class Module:
     """Sets an attribute on this Module.
     
     We overload setattr solely to support pythonic naming via assignment of 
-    submodules in the special setup() function::
+    submodules in the special :meth:`setup` function::
 
       self.submodule_name = MyModule(...)
 
@@ -505,12 +507,13 @@ class Module:
     """
     is_dataclass_attr = name in self.__dataclass_fields__ and self.__dataclass_fields__[name].init  # pytype: disable=attribute-error
     
-    if not self._state.in_setup and not is_dataclass_attr:
+    if not self._state.in_setup and self._state.is_initialized:
       # Raises a TypeError just like frozen python dataclasses.
-      raise TypeError("Module instance is frozen outside of setup method.")
+      raise errors.SetAttributeFrozenModuleError(self.__class__.__name__, name, 
+                                                 val)
     if is_dataclass_attr:
       if self._state.in_setup:
-        raise TypeError("Module construction attributes are frozen.")
+        raise errors.SetAttributeInModuleSetupError()
       object.__setattr__(self, name, val)
     # Submodules are being defined and attached in setup()
     else:
@@ -526,7 +529,7 @@ class Module:
       return self.__dict__[name]
     else:
       raise AttributeError(
-          f"'{self.__class__.__name__}' object has no attribute '{name}'")
+          f'"{self.__class__.__name__}" object has no attribute "{name}"')
 
   def __dir__(self) -> List[str]:
     """Call setup() before listing attributes."""
@@ -534,7 +537,7 @@ class Module:
     return object.__dir__(self)  # pytype: disable=attribute-error
 
   def __post_init__(self):
-    _check_omnistaging()
+    # DO NOT REMOVE - Marker for internal logging.
     # In dataclasses, __init__ is overridden to process dataclass arguments,
     # and __post_init__ is called immediately afterwards. Here, depending on the
     # type of `parent` passed to initialize the Module, we either defer 
@@ -560,9 +563,7 @@ class Module:
       if self.parent._state.in_setup and self.name is None:  # pytype: disable=attribute-error
         return
       if not self.parent._initialization_allowed:
-        raise ValueError(
-            'Submodules must be defined in `setup()` or in a method wrapped '
-            'in `@compact`')
+        raise errors.AssignSubModuleError(self.__class__.__name__)
       # Autonaming of submodules.
       if self.name is None:  # pytype: disable=attribute-error
         prefix = f"{self.__class__.__name__}"
@@ -570,11 +571,8 @@ class Module:
         self.name = f"{prefix}_{cursor}"
         self.parent._state.autoname_cursor[prefix] = cursor + 1
       if self.parent._name_taken(self.name, self):
-        raise ValueError(
-            f"A variable of name {self.name} exists already, or "
-            f"trying to share submodule {self.__class__.__name__} by name "
-            f"{self.name}. To share submodules, store module instances as a"
-            f" Python object or as an attribute on self and reuse.")
+        parent_class = self.parent.__class__.__name__
+        raise errors.NameInUseError('submodule', self.name, parent_class)
       self.parent._state.children[self.name] = self
       object.__setattr__(self, 'scope', self.parent.scope.push(self.name))
 
@@ -584,6 +582,8 @@ class Module:
     else:
       raise ValueError("parent must be None, Module or Scope")
 
+    self._state.is_initialized = True
+
   def __repr__(self):
     return _module_repr(self)
 
@@ -700,6 +700,16 @@ class Module:
     for a shorthand way to define read-only variables in the "params"
     collection.
 
+    Contrary to :meth:`param`, all arguments passing using `init_fn` should be
+    passed on explictly::
+
+      key = self.make_rng('stats')
+      mean = self.variable('stats', 'mean', lecun_normal(), key, (2, 2))
+
+    In the example above, the function `lecun_normal` expects two arguments:
+    `key` and `shape`, and both have to be passed on. The PRNG for `stats` has
+    to be provided explicitly when calling :meth:`init` and :meth:`apply`.
+
     Args:
       col: The variable collection name.
       name: The variable name.
@@ -717,8 +727,7 @@ class Module:
           'Variables must be initialized in `setup()` or in a method '
           'wrapped in `@compact`')
     if self._name_taken(name):
-      raise ValueError(
-          f'Name {name} already in use in {self.__class__.__name__}.')
+      raise errors.NameInUseError('variable', name, self.__class__.__name__)
     v = self.scope.variable(col, name, init_fn, *init_args)
     self._state.children[name] = col
     return v
@@ -729,6 +738,16 @@ class Module:
     Parameters are read-only variables in the collection named "params". See
     :mod:`flax.core.variables` for more details on variables.
 
+    The first argument of `init_fn` is assumed to be a PRNG key, which is
+    provided automatically and does not have to be passed using `init_args`::
+
+      mean = self.param('mean', lecun_normal(), (2, 2))
+
+    In the example above, the function `lecun_normal` expects two arguments:
+    `key` and `shape`, but only `shape` has to be provided explicitly; `key`
+    is set automatically using the PRNG for `params` that is passed when
+    initializing the module using :meth:`init`.
+
     Args:
       name: The parameter name.
       init_fn: The function that will be called to compute the initial value
@@ -744,8 +763,7 @@ class Module:
           'Parameters must be initialized in `setup()` or in a method '
           'wrapped in `@compact`')
     if self._name_taken(name):
-      raise ValueError(
-          f'Name {name} already in use in {self.__class__.__name__}.')
+      raise errors.NameInUseError('param', name, self.__class__.__name__)
     v = self.scope.param(name, init_fn, *init_args)
     self._state.children[name] = 'params'
     return v
@@ -790,35 +808,95 @@ class Module:
       raise ValueError("Can't use RNGs on unbound modules")
     return self.scope.make_rng(name)
 
+  def bind(self, variables: VariableDict, *args, rngs: RNGSequences = None,
+           mutable: CollectionFilter = False):
+    """Creates an interactive Module instance by binding variables and RNGs.
+
+    ``bind`` provides an "interactive" instance of a Module directly without
+    transforming a function with ``apply``. This is particulary useful for debugging
+    and interactive use cases like notebooks where a function would limit the ability
+    split up code into different cells.
+
+    Once the variables (and optionally RNGs) are bound to a ``Module`` it becomes a
+    stateful object. Note that idiomatic JAX is functional and therefore an interactive
+    instance does not mix well well with vanilla JAX APIs. ``bind()`` should only be used
+    for interactive experimentation, and in all other cases we strongly encourage
+    to use ``apply()`` instead.
+
+    Example::
+
+      class AutoEncoder(nn.Module):
+        def setup(self):
+          self.encoder = nn.Dense(3)
+          self.decoder = nn.Dense(5)
+      
+      ae = AutoEncoder()
+      model = ae.bind(variables)
+      z = model.encode(x)
+      x_reconstructed = model.decode(z)
+
+
+    Args:
+       variables: A dictionary containing variables keyed by variable
+        collections. See :mod:`flax.core.variables` for more details
+        about variables.
+      rngs: a dict of PRNGKeys to initialize the PRNG sequences.
+      mutable: Can be bool, str, or list. Specifies which collections should be
+               treated as mutable: ``bool``: all/no collections are mutable.
+               ``str``: The name of a single mutable collection. ``list``: A
+               list of names of mutable collections.
+    Returns:
+      A copy of this instance with bound variables and RNGs.
+    """
+    scope = core.bind(variables, rngs=rngs, mutable=mutable)
+    return self.clone(parent=scope)
+
   def apply(self, variables: VariableDict, *args, rngs: RNGSequences = None,
             method: Callable[..., Any] = None, 
-            mutable: Union[bool, str, Sequence[str]] = False,
+            mutable: CollectionFilter = False,
             capture_intermediates: Union[bool, Callable[['Module', str], bool]] = False,
             **kwargs) -> Union[Any, Tuple[Any, FrozenVariableDict]]:
     """Applies a module method to variables and returns output and modified variables.
 
     Note that `method` should be set if one would like to call `apply` on a
-    different class method than `_call__`. For instance, suppose a Transformer
-    modules has a method called `encode`, then the following calls `apply` on
-    that method::
+    different class method than ``__call__``. For instance, suppose a
+    Transformer modules has a method called `encode`, then the following calls
+    `apply` on that method::
 
-      model = models.Transformer(config)
-      encoded = model.apply({'params': params}, inputs, method=model.encode)
+      model = Transformer()
+      encoded = model.apply({'params': params}, x, method=Transformer.encode)
+  
+    If a function instance is provided, the unbound function is used. For 
+    instance, the example below is equivalent to the one above::
+
+      encoded = model.apply({'params': params}, x, method=model.encode)
+
+    Note ``method`` can also be a function that is not defined in
+    ``Transformer``. In that case, the function should have at least one
+    argument representing an instance of the Module class::
+
+      def other_fn(instance, ...):
+        instance.some_module_attr(...)
+        ...
+
+      model.apply({'params': params}, x, method=other_fn)
 
     Args:
       variables: A dictionary containing variables keyed by variable
         collections. See :mod:`flax.core.variables` for more details
         about variables.
-      rngs: The rngs for the variable collections.
-      method: The literal name of a method in this class. If provided, applies
-        this method. If not provided, applies the ``__call__`` method.
+      rngs: a dict of PRNGKeys to initialize the PRNG sequences.
+        The "params" PRNG sequence is used to initialize parameters.
+      method: A function to call apply on. This is generally a function in the
+        module. If provided, applies this method. If not provided, applies the
+        ``__call__`` method of the module.
       mutable: Can be bool, str, or list. Specifies which collections should be
                treated as mutable: ``bool``: all/no collections are mutable.
                ``str``: The name of a single mutable collection. ``list``: A
                list of names of mutable collections.
       capture_intermediates: If `True`, captures intermediate return values
         of all Modules inside the "intermediates" collection. By default only
-        the return values of all `__call__` methods are stored. A function can
+        the return values of all ``__call__`` methods are stored. A function can
         be passed to change the filter behavior. The filter function takes
         the Module instance and method name and returns a bool indicating
         whether the output of that method invocation should be stored.
@@ -828,19 +906,12 @@ class Module:
       of the modified collections.
     """
     if method is None:
-      method = self.__class__.__call__
-    else:
-      method = _get_unbound_fn(method)
-    fn = lambda scope: method(self.clone(parent=scope), *args, **kwargs)
-    if capture_intermediates is True:
-      capture_intermediates = capture_call_intermediates
-    if capture_intermediates:
-      mutable = union_filters(mutable, 'intermediates')
-    _context.capture_stack.append(capture_intermediates)
-    try:
-      return apply(fn, mutable=mutable)(variables, rngs=rngs)
-    finally:
-      _context.capture_stack.pop()
+      method = self.__call__
+    method = _get_unbound_fn(method)
+    return apply(
+        method, self,
+        mutable=mutable, capture_intermediates=capture_intermediates
+    )(variables, *args, **kwargs, rngs=rngs)
     
 
   def init_with_output(self, rngs: Union[PRNGKey, RNGSequences], *args,
@@ -857,7 +928,10 @@ class Module:
       collections.
     """
     if not isinstance(rngs, dict):
-      assert rngs.shape == (2,)
+      if rngs.shape != (2,):
+        raise errors.InvalidRngError(
+            'RNGs should be of shape (2,) in Module '
+            f'{self.__class__.__name__}, but rngs are: {rngs}')
       rngs = {'params': rngs}
     return self.apply(
         {}, *args, rngs=rngs, method=method, mutable=True, **kwargs)
@@ -867,12 +941,12 @@ class Module:
            **kwargs) -> FrozenVariableDict:
     """Initializes a module method with variables and returns modified variables.
 
-    Jitting `init` initializes a model lazily using only the shapes of the 
-    provided arguments, and avoids computing the forward pass with actual 
+    Jitting `init` initializes a model lazily using only the shapes of the
+    provided arguments, and avoids computing the forward pass with actual
     values. Example::
 
       jit_init = jax.jit(SomeModule.init)
-      jit_init(rng, jnp.ones(input_shape, jnp.float32))      
+      jit_init(rng, jnp.ones(input_shape, jnp.float32))
 
     Args:
       rngs: The rngs for the variable collections.
@@ -980,7 +1054,7 @@ def merge_param(name: str, a: Optional[T], b: Optional[T]) -> T:
   """Merges construction and call time argument.
 
   This is a utility for supporting the pattern where a Module hyper parameter
-  can be passed to `__init__` or `__call__`.
+  can be passed to ``__init__`` or ``__call__``.
 
   Example::
 
@@ -1011,61 +1085,144 @@ def merge_param(name: str, a: Optional[T], b: Optional[T]) -> T:
     return a
 
 
-  # THE PART BELOW IS STILL UNDER DEVELOPMENT, PLEASE IGNORE
-  # ===========================================================
-  # @contextmanager
-  # def mutate(self, mutable=True, **updates):
-  #   cloned = self.clone(**updates)
-  #   try:
-  #     cloned.scope._variables = _unfreeze_variables(
-  #         cloned.scope._variables, mutable)
-  #     yield cloned
-  #   finally:
-  #     cloned.scope._variables = freeze(cloned.scope._variables)
+def apply(fn: Callable[..., Any], module: Module,
+          mutable: CollectionFilter = False,
+          capture_intermediates: Union[bool, Callable[[Module, str], bool]] = False) -> Callable[..., Any]:
+  """Creates an apply function to call ``fn`` with a bound module.
 
-  # def initialized(self, rngs, *args, method='__call__', **kwargs):
-  #   if self.parent is not None:
-  #     raise ValueError("Pattern for initialized is "
-  #                      "`Module(parent=None, ...attrs...).initialized(...)`")
-  #   scope = Scope(variables={}, rngs=rngs)
-  #   with self.mutate(parent=scope) as initialized:
-  #     if method is not None:
-  #       getattr(initialized, method)(*args, **kwargs)
-  #   return initialized
+  Unlike ``Module.apply`` this function returns a new function with the signature
+  ``(variables, *args, rngs=None, **kwargs) -> T`` where `T` is the return type
+  of ``fn``. If ``mutable`` is not ``False`` the return type is a tuple where the
+  second item is a ``FrozenDict`` with the mutated variables.
 
-  # @property
-  # def variables(self):
-  #   """Get a view of Module variables with easy dot-syntax navigation."""
-  #   return DotGetter(self.scope.variables())
+  The apply function that is returned can be directly composed with
+  JAX transformations like ``jax.jit``::
 
-  # def __getattr__(self, name):
-  #   # Used for easy colab/jupyter introspection, and to provide a
-  #   # consistent top-level interface to self.<attr> for both simple
-  #   # and multi-method modules.
-  #   if name in self.children:
-  #     val = self.children[name]
-  #     if isinstance(val, str):  # variable
-  #       return self.variables[val][name]
-  #     else:  # submodule
-  #       val.scope = self.scope.push(name)
-  #       self.scope.reservations.remove(name)
-  #       return val
-  #   else:
-  #     raise AttributeError(
-  #         f"'{self.__class__.__name__}' object has no attribute '{name}'")
+    def f(foo, x):
+      z = foo.encode(x)
+      y = foo.decode(z)
+      # ...
+      return y
+    
+    foo = Foo()
+    f_jitted = jax.jit(nn.apply(f, foo))
+    f_jitted(variables, x)
 
-  # def __dir__(self):
-  #   return list(self.children.keys()) + object.__dir__(self)
+  Args:
+    fn: The function that should be applied. The first argument passed will
+      be an module instance of the ``module`` with variables and RNGs bound
+      to it.
+    module: The ``Module`` that will be used to bind variables and RNGs to.
+      The ``Module`` passed as the first argument to ``fn`` will be a clone
+      of module.
+    mutable: Can be bool, str, or list. Specifies which collections should be
+      treated as mutable: ``bool``: all/no collections are mutable.
+      ``str``: The name of a single mutable collection. ``list``: A
+      list of names of mutable collections.
+    capture_intermediates: If `True`, captures intermediate return values
+      of all Modules inside the "intermediates" collection. By default only
+      the return values of all `__call__` methods are stored. A function can
+      be passed to change the filter behavior. The filter function takes
+      the Module instance and method name and returns a bool indicating
+      whether the output of that method invocation should be stored.
+  Returns:
+    The apply function wrapping ``fn``.
+  """
+  @functools.wraps(fn)
+  def scope_fn(scope, *args, **kwargs):
+    _context.capture_stack.append(capture_intermediates)
+    try:
+      return fn(module.clone(parent=scope), *args, **kwargs)
+    finally:
+      _context.capture_stack.pop()
 
-  # TODO: Should this be what `clone` always does if you don't pass in an explicit
-  # parent?
-  # def detached(self):
-  #   return self.clone(parent=None)
+  if capture_intermediates is True:
+    capture_intermediates = capture_call_intermediates
+  if capture_intermediates:
+    mutable = union_filters(mutable, 'intermediates')
+  return core.apply(scope_fn, mutable=mutable)
 
-  # # TODO: Consider whether this is a helpful abstraction, and think about naming.
-  # # See its use in design_test/linen/weight_std.py
-  # def materialized(self, variables={}, rngs={}):
-  #   assert self.scope is None, ("Can't attach a module twice."
-  #                               " Maybe you want to clone first?")
-  #   return self.clone(parent=Scope(variables, rngs))
 
+def init_with_output(fn: Callable[..., Any], module: Module,
+                     mutable: CollectionFilter = True) -> Callable[..., Tuple[Any, FrozenVariableDict]]:
+  """Creates an init function to call ``fn`` with a bound module that also returns the function outputs.
+
+  Unlike ``Module.init_with_output`` this function returns a new function with the signature
+  ``(rngs, *args, **kwargs) -> (T, variables)`` where `T` is the return type of ``fn``.
+  The rngs can be a dict of PRNGKeys or a single ```PRNGKey`` which is
+  equivalant to passing a dict with one PRNGKey with the name "params".
+
+  The init function that is returned can be directly composed with
+  JAX transformations like ``jax.jit``::
+
+    def f(foo, x):
+      z = foo.encode(x)
+      y = foo.decode(z)
+      # ...
+      return y
+    
+    foo = Foo()
+    f_jitted = jax.jit(nn.init_with_output(f, foo))
+    y, variables = f_jitted(rng, x)
+
+  Args:
+    fn: The function that should be applied. The first argument passed will
+      be an module instance of the ``module`` with variables and RNGs bound
+      to it.
+    module: The ``Module`` that will be used to bind variables and RNGs to.
+      The ``Module`` passed as the first argument to ``fn`` will be a clone
+      of module.
+    mutable: Can be bool, str, or list. Specifies which collections should be
+      treated as mutable: ``bool``: all/no collections are mutable.
+      ``str``: The name of a single mutable collection. ``list``: A
+      list of names of mutable collections.
+  Returns:
+    The init function wrapping ``fn``.
+  """
+  @functools.wraps(fn)
+  def scope_fn(scope, *args, **kwargs):
+    return fn(module.clone(parent=scope), *args, **kwargs)
+  return core.init(scope_fn, mutable=mutable)
+
+
+def init(fn: Callable[..., Any], module: Module,
+         mutable: CollectionFilter = True) -> Callable[..., FrozenVariableDict]:
+  """Creates an init function to call ``fn`` with a bound module.
+
+  Unlike ``Module.init`` this function returns a new function with the signature
+  ``(rngs, *args, **kwargs) -> variables``.
+  The rngs can be a dict of PRNGKeys or a single ```PRNGKey`` which is
+  equivalant to passing a dict with one PRNGKey with the name "params".
+
+  The init function that is returned can be directly composed with
+  JAX transformations like ``jax.jit``::
+
+    def f(foo, x):
+      z = foo.encode(x)
+      y = foo.decode(z)
+      # ...
+      return y
+    
+    foo = Foo()
+    f_jitted = jax.jit(nn.init(f, foo))
+    variables = f_jitted(rng, x)
+
+  Args:
+    fn: The function that should be applied. The first argument passed will
+      be an module instance of the ``module`` with variables and RNGs bound
+      to it.
+    module: The ``Module`` that will be used to bind variables and RNGs to.
+      The ``Module`` passed as the first argument to ``fn`` will be a clone
+      of module.
+    mutable: Can be bool, str, or list. Specifies which collections should be
+      treated as mutable: ``bool``: all/no collections are mutable.
+      ``str``: The name of a single mutable collection. ``list``: A
+      list of names of mutable collections.
+  Returns:
+    The init function wrapping ``fn``.
+  """
+  init_fn = init_with_output(fn, module, mutable)
+  @functools.wraps(init_fn)
+  def init_wrapper(*args, **kwargs):
+    return init_fn(*args, **kwargs)[1]
+  return init_wrapper
