@@ -270,6 +270,8 @@ class Module(metaclass=_ModuleMeta):
   Functional modules."""
 
   def __new__(cls, *args, name=None, **kwargs):
+    warnings.warn("The `flax.nn` module is Deprecated, use `flax.linen` instead. Learn more and find an upgrade guide at https://github.com/google/flax/blob/master/flax/linen/README.md", DeprecationWarning)
+    # DO NOT REMOVE - Marker for internal logging.
     if not _module_stack:
       raise ValueError('A Module should only be instantiated directly inside'
                        ' another module.')
@@ -953,7 +955,8 @@ class Model:
   """DEPRECATION WARNING:
   The `flax.nn` module is Deprecated, use `flax.linen` instead. 
   Learn more and find an upgrade guide at 
-  https://github.com/google/flax/blob/master/flax/linen/README.md"
+  https://github.com/google/flax/blob/master/flax/linen/README.md
+
   A Model contains the model parameters, state and definition."""
 
   module: Type[Module] = struct.field(pytree_node=False)
