@@ -1,4 +1,4 @@
-# Copyright 2020 The Flax Authors.
+# Copyright 2021 The Flax Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import numpy as onp
+import numpy as np
 
 from .. import struct
 
@@ -21,7 +21,7 @@ from .base import OptimizerDef
 
 @struct.dataclass
 class _GradientDescentHyperParams:
-  learning_rate: onp.ndarray
+  learning_rate: np.ndarray
 
 
 class GradientDescent(OptimizerDef):
